<?php
if (!defined('ABSPATH'))
    exit;

if (!class_exists('berqCache')) {
    class berqCache
    {
        public $cache_file;
        public $max_page_per_batch = 20;

        // Special Thanks: https://github.com/mpchadwick/tracking-query-params-registry/blob/master/_data/params.csv
        public $ignored_query_params = [
            'Browser',
            'C',
            'GCCON',
            'MCMP',
            'MarketPlace',
            'PD',
            'Refresh',
            'Sens',
            'ServiceVersion',
            'Source',
            'Topic',
            '__WB_REVISION__',
            '__cf_chl_jschl_tk__',
            '__d',
            '__hsfp',
            '__hssc',
            '__hstc',
            '__s',
            '_branch_match_id',
            '_bta_c',
            '_bta_tid',
            '_com',
            '_escaped_fragment_',
            '_ga',
            '_ga-ft',
            '_gl',
            '_hsmi',
            '_ke',
            '_kx',
            '_paged',
            '_sm_byp',
            '_sp',
            '_szp',
            '_thumbnail_id',
            '3x',
            'a',
            'a_k',
            'ac',
            'acpage',
            'action-box',
            'action_object_map',
            'action_ref_map',
            'action_type_map',
            'activecampaign_id',
            'ad',
            'ad_frame_full',
            'ad_frame_root',
            'ad_name',
            'adclida',
            'adid',
            'adlt',
            'adsafe_ip',
            'adset_name',
            'advid',
            'aff_sub2',
            'afftrack',
            'afterload',
            'ak_action',
            'alt_id',
            'am',
            'amazingmurphybeds',
            'amp;',
            'amp;amp',
            'amp;amp;amp',
            'amp;amp;amp;amp',
            'amp;utm_campaign',
            'amp;utm_medium',
            'amp;utm_source',
            'amp%3Butm_content',
            'ampStoryAutoAnalyticsLinker',
            'ampstoryautoanalyticslinke',
            'an',
            'ap',
            'ap_id',
            'apif',
            'apipage',
            'as_occt',
            'as_q',
            'as_qdr',
            'askid',
            'atFileReset',
            'atfilereset',
            'aucid',
            'auct',
            'audience',
            'author',
            'awt_a',
            'awt_l',
            'awt_m',
            'b2w',
            'back',
            'bannerID',
            'blackhole',
            'blockedAdTracking',
            'blog-reader-used',
            'blogger',
            'body',
            'br',
            'bsft_aaid',
            'bsft_clkid',
            'bsft_eid',
            'bsft_ek',
            'bsft_lx',
            'bsft_mid',
            'bsft_mime_type',
            'bsft_tv',
            'bsft_uid',
            'bvMethod',
            'bvTime',
            'bvVersion',
            'bvb64',
            'bvb64resp',
            'bvplugname',
            'bvprms',
            'bvprmsmac',
            'bvreqmerge',
            'cacheburst',
            'campaign',
            'campaign_id',
            'campaign_name',
            'campid',
            'catablog-gallery',
            'channel',
            'checksum',
            'ck_subscriber_id',
            'cmplz_region_redirect',
            'cmpnid',
            'cn-reloaded',
            'code',
            'comment',
            'content_ad_widget',
            'cost',
            'cr',
            'crl8_id',
            'crlt.pid',
            'crlt_pid',
            'crrelr',
            'crtvid',
            'ct',
            'cuid',
            'daksldlkdsadas',
            'dcc',
            'dfp',
            'dm_i',
            'domain',
            'dosubmit',
            'dsp_caid',
            'dsp_crid',
            'dsp_insertion_order_id',
            'dsp_pub_id',
            'dsp_tracker_token',
            'dt',
            'dur',
            'durs',
            'e',
            'ee',
            'ef_id',
            'el',
            'emailID',
            'env',
            'epik',
            'erprint',
            'et_blog',
            'exch',
            'externalid',
            'fb_action_ids',
            'fb_action_types',
            'fb_ad',
            'fb_source',
            'fbclid',
            'fbzunique',
            'fg-aqp',
            'fireglass_rsn',
            'firstName',
            'fo',
            'fp_sid',
            'fpa',
            'fref',
            'fs',
            'furl',
            'fwp_lunch_restrictions',
            'ga_action',
            'gclid',
            'gclsrc',
            'gdffi',
            'gdfms',
            'gdftrk',
            'gf_page',
            'gidzl',
            'goal',
            'gooal',
            'gpu',
            'gtVersion',
            'haibwc',
            'hash',
            'hc_location',
            'hemail',
            'hid',
            'highlight',
            'hl',
            'home',
            'hsa_acc',
            'hsa_ad',
            'hsa_cam',
            'hsa_grp',
            'hsa_kw',
            'hsa_mt',
            'hsa_net',
            'hsa_src',
            'hsa_tgt',
            'hsa_ver',
            'ias_campId',
            'ias_chanId',
            'ias_dealId',
            'ias_dspId',
            'ias_impId',
            'ias_placementId',
            'ias_pubId',
            'ical',
            'ict',
            'ie',
            'igshid',
            'im',
            'ipl',
            'jw_start',
            'jwsource',
            'k',
            'key1',
            'key2',
            'klaviyo',
            'ksconf',
            'ksref',
            'l',
            'label',
            'lang',
            'ldtag_cl',
            'level1',
            'level2',
            'level3',
            'level4',
            'limit',
            'lng',
            'load_all_comments',
            'lt',
            'ltclid',
            'ltd',
            'lucky',
            'm',
            'm?sales_kw',
            'matomo_campaign',
            'matomo_cid',
            'matomo_content',
            'matomo_group',
            'matomo_keyword',
            'matomo_medium',
            'matomo_placement',
            'matomo_source',
            'max-results',
            'mc_cid',
            'mc_eid',
            'mdrv',
            'mediaserver',
            'memset',
            'mibextid',
            'mkcid',
            'mkevt',
            'mkrid',
            'mkwid',
            'mkt_tok',
            'ml_subscriber',
            'ml_subscriber_hash',
            'mobileOn',
            'mode',
            'moderation-hash',
            'modernpatio',
            'month',
            'msID',
            'msclkid',
            'msg',
            'mtm_campaign',
            'mtm_cid',
            'mtm_content',
            'mtm_group',
            'mtm_keyword',
            'mtm_medium',
            'mtm_placement',
            'mtm_source',
            'murphybedstoday',
            'mwprid',
            'n',
            'name',
            'native_client',
            'navua',
            'nb',
            'nb_klid',
            'nowprocketcache',
            'o',
            'okijoouuqnqq',
            'org',
            'pa_service_worker',
            'partnumber',
            'pcmtid',
            'pcode',
            'pcrid',
            'pfstyle',
            'phrase',
            'pid',
            'piwik_campaign',
            'piwik_keyword',
            'piwik_kwd',
            'pk_campaign',
            'pk_keyword',
            'pk_kwd',
            'placement',
            'plat',
            'platform',
            'playsinline',
            'position',
            'pp',
            'pr',
            'preview',
            'preview_id',
            'preview_nonce',
            'prid',
            'print',
            'q',
            'q1',
            'qsrc',
            'r',
            'rd',
            'rdt_cid',
            'redig',
            'redir',
            'ref',
            'reftok',
            'relatedposts_hit',
            'relatedposts_origin',
            'relatedposts_position',
            'remodel',
            'replytocom',
            'rest_route',
            'reverse-paginate',
            'rid',
            'rnd',
            'rndnum',
            'robots_txt',
            'rq',
            'rsd',
            's_kwcid',
            'sa',
            'safe',
            'said',
            'sales_cat',
            'sales_kw',
            'sb_referer_host',
            'scrape',
            'script',
            'scrlybrkr',
            'search',
            'sellid',
            'sersafe',
            'sfn_data',
            'sfn_trk',
            'sfns',
            'sfw',
            'sha1',
            'share',
            'shared',
            'showcomment',
            'showComment',
            'si',
            'sid',
            'sid1',
            'sid2',
            'sidewalkShow',
            'sig',
            'site',
            'site_id',
            'siteid',
            'slicer1',
            'slicer2',
            'source',
            'spref',
            'spvb',
            'sra',
            'src',
            'srk',
            'srp',
            'ssp_iabi',
            'ssts',
            'stylishmurphybeds',
            'subId1 ',
            'subId2 ',
            'subId3',
            'subid',
            'swcfpc',
            'tail',
            'teaser',
            'test',
            'timezone',
            'toWww',
            'triplesource',
            'trk_contact',
            'trk_module',
            'trk_msg',
            'trk_sid',
            'tsig',
            'turl',
            'u',
            'unapproved',
            'up_auto_log',
            'upage',
            'updated-max',
            'uptime',
            'us_privacy',
            'usegapi',
            'userConsent',
            'usqp',
            'utm',
            'utm_campa',
            'utm_campaign',
            'utm_content',
            'utm_expid',
            'utm_id',
            'utm_medium',
            'utm_reader',
            'utm_referrer',
            'utm_source',
            'utm_sq',
            'utm_ter',
            'utm_term',
            'v',
            'vc',
            'vf',
            'vgo_ee',
            'vp',
            'vrw',
            'vz',
            'wbraid',
            'webdriver',
            'wing',
            'wpdParentID',
            'wpmp_switcher',
            'wref',
            'wswy',
            'wtime',
            'x',
            'zMoatImpID',
            'zarsrc',
            'zeffdn'
        ];

        function __construct()
        {
            add_filter( 'berqwp_ignored_urls_params', [$this, 'ingore_tracking_params'] );

            $this->cache_file = berqwp_current_page_cache_file();

            // clear cache after migration
            if (get_option('berqwp_site_url', home_url()) !== home_url()) {

                if (get_option('berqwp_site_url', home_url()) !== null) {
                    global $berq_log;
                    $berq_log->info('Home url change detected, flushing all cache');
                }

                $this->delete_cache_files();
                update_option('berqwp_site_url', home_url());
            }


            // Add clear cache link to admin bar
            add_action('admin_bar_menu', [$this, 'add_clear_cache_link_to_admin_bar'], 999);

            // Flush cache
            add_action('admin_post_clear_cache', [$this, 'handle_clear_cache_action']);

            // Purge a page cache
            add_action('admin_post_berq_purge_page', [$this, 'handle_berq_purge_page_action']);

            // Clear page cache on update
            add_action('save_post', [$this, 'clear_cache_on_post_udpate'], 10, 3);

            // Automattic cache warmup
            add_action('init', [$this, 'warmup_queue']);
            add_action('warmup_cache_by_slug', [$this, 'handle_warmup_cache_by_slug']);
            add_action('admin_notices', [$this, 'cache_warmup_admin_notice']);


            add_action('wp_loaded', [$this, 'berqwp_warmup_cache_all_pages']);


        }

        function ingore_tracking_params($tracking_params) {
            $tracking_params = array_merge($tracking_params, $this->ignored_query_params);
            return $tracking_params;
        }

        function berqwp_warmup_cache_all_pages()
        {

            if (isset($_GET['run_warmup'])) {

                global $berq_log;
                $berq_log->info('* * * * * * * * * * * * *');
                $berq_log->info('* Manual Cache Warmup');
                $berq_log->info('* * * * * * * * * * * * *');

                // Get all published pages and posts
                $args = array(
                    'post_type' => array('page'),
                    'post_status' => 'publish',
                    'posts_per_page' => -1, // Get all posts
                );
                $query = new WP_Query($args);

                // API endpoint URL
                $api_endpoint = 'https://boost.berqwp.com/photon/';

                if (get_site_url() == 'http://berq-test.local') {
                    $api_endpoint = 'http://dev-berqwp.local/photon/';
                }

                $httpMulti = new bwp_multi_http();


                // Loop through each post/page and warm up the cache
                if ($query->have_posts()) {
                    while ($query->have_posts()) {
                        $query->the_post();
                        $url = trailingslashit(get_permalink()); // Get the permalink of the post/page

                        // $httpMulti->addRequest('GET', $url);
                        $slug = str_replace(home_url(), '', $url);
                        $post_data = berqwp_get_page_params($slug, true);

                        $httpMulti->addRequest('POST', $api_endpoint, $post_data);

                        $berq_log->info("Requesting cache for $slug");
                    }
                }

                $responses = $httpMulti->execute();
                wp_reset_postdata();

            }
        }

        function warmup_queue()
        {
            if (!file_exists(ABSPATH . 'wp-content/advanced-cache.php')) {
                // Specify the drop-in file path
                $dropin_file = ABSPATH . 'wp-content/advanced-cache.php';

                // Dynamically create the drop-in file
                $dropin_content = <<<EOD
<?php
/**
 * BerqWP Advanced Cache Drop-in
 */

add_action('wp', function () {
    if (!is_admin()) {
        if (class_exists('berqCache')) {
            \$berqCache = new berqCache();
            \$berqCache->html_cache();
        }
    }
});
EOD;

                // Write the drop-in content to the file, replacing any existing file
                file_put_contents($dropin_file, $dropin_content);

                // Enable object cache in wp-config.php
                berqwp_enable_object_cache(true);

                if (function_exists('wp_cache_flush')) {
                    wp_cache_flush(); // Clear the entire object cache.
                }
            }

            // if (!get_transient('berqwp_doing_cache_warmup')) {
            if (false === as_has_scheduled_action('warmup_cache_by_slug')) {
                $post_types = get_option('berqwp_post_type_names');

                $args = array(
                    'post_type' => $post_types,
                    'posts_per_page' => -1,
                    'fields' => 'ids',
                    'post_status' => 'publish',
                );

                $query = new WP_Query($args);

                // Calculate the total number of pages
                $total_posts = count($query->posts);

                $posts_per_batch = $this->max_page_per_batch;
                $total_pages = ceil($total_posts / $posts_per_batch);


                // Schedule the events
                for ($page = 1; $page <= $total_pages; $page++) {
                    as_schedule_single_action(time() + (80 * $page), 'warmup_cache_by_slug', array($page));
                }
            }
            // }


        }

        function can_send_warmup_request()
        {
            $requests_sent = get_transient('berqwp_warmup_requests_count');

            // If we haven't set the rate limiting transient yet, or if we're still below our rate, return true.
            if (false === $requests_sent || $requests_sent < BERQWP_MAX_WARMUP_REQUESTS) {
                return true;
            }

            return false;
        }

        function update_warmup_request_count()
        {
            $requests_sent = get_transient('berqwp_warmup_requests_count');

            if (false === $requests_sent) {
                // If this is the first request in our window, set the transient with a timeout.
                set_transient('berqwp_warmup_requests_count', 1, BERQWP_WARMUP_RATE_LIMIT_WINDOW);
            } else {
                // Otherwise, just increment the counter.
                set_transient('berqwp_warmup_requests_count', ++$requests_sent, BERQWP_WARMUP_RATE_LIMIT_WINDOW);
            }
        }



        function handle_warmup_cache_by_slug($page)
        {
            // error_log('-page-' . $page);
            if ($this->can_send_warmup_request()) {
                // error_log('page-' . $page);
                $this->warmup_cache($page);
                $this->update_warmup_request_count();
            } else {
                // You've hit the rate limit.
                $retry_after_seconds = 200; // Retry after 10 minutes. You can adjust this time as needed.
                // as_schedule_single_action(time() + $retry_after_seconds, 'warmup_cache_by_slug', array($page));
            }
        }



        function clear_cache_on_post_udpate($post_id, $post, $update)
        {
            // If this is just a revision, don't run the function.
            if (wp_is_post_revision($post_id)) {
                return;
            }

            $post_url = get_permalink($post_id);
            $slug = str_replace(home_url(), '', $post_url);
            $cache_key = md5($slug);
            $cache_directory = optifer_cache . '/html/';
            $cache_file = $cache_directory . $cache_key . '.html';

            global $berq_log;
            $berq_log->info("Post updated, deleting cache for $post_url");

            if (file_exists($cache_file)) {
                unlink($cache_file);
            }

        }

        function flatten_array($array)
        {
            $result = array();
            foreach ($array as $item) {
                if (is_array($item)) {
                    $result = array_merge($result, $this->flatten_array($item));
                } else {
                    $result[] = $item;
                }
            }
            return $result;
        }

        function handle_berq_purge_page_action()
        {
            // Check if the user has the necessary nonce and the action matches
            if (isset($_GET['action']) && $_GET['action'] === 'berq_purge_page' && wp_verify_nonce($_GET['_wpnonce'], 'berq_purge_page_action')) {

                $slug = sanitize_text_field($_GET['uri']);
                $cache_file = optifer_cache . '/html/' . md5($slug) . '.html';

                if (function_exists('wp_cache_flush')) {
                    wp_cache_flush(); // Clear the entire object cache.
                }

                if (is_file($cache_file)) {
                    unlink($cache_file);
                }

                // Redirect back to the referring page after clearing the cache
                wp_safe_redirect(wp_get_referer());
                exit;
            }
        }

        function handle_clear_cache_action()
        {
            // Check if the user has the necessary nonce and the action matches
            if (isset($_GET['action']) && $_GET['action'] === 'clear_cache' && wp_verify_nonce($_GET['_wpnonce'], 'clear_cache_action')) {
                // Call your custom cache clearing function here
                $this->delete_cache_files();

                if (function_exists('wp_cache_flush')) {
                    wp_cache_flush(); // Clear the entire object cache.
                }

                set_transient('berq_cache_cleared_notice', 'true', 60);

                $redirect_url = add_query_arg('berq_clear_cache', '', wp_get_referer());

                // Redirect back to the referring page after clearing the cache
                wp_safe_redirect($redirect_url);
                exit;
            }
        }

        function delete_cache_files()
        {
            global $berq_log;
            $berq_log->info("Flushing all cache.");

            // Define the cache directory
            $cache_directory = optifer_cache . '/html/';

            // Delete all cache files within the directory
            berqwp_unlink_recursive($cache_directory);

            // $files = glob($cache_directory . '*.html');
            // foreach ($files as $file) {
            //     if (is_file($file)) {
            //         unlink($file);
            //     }
            // }

            delete_transient('berqwp_warmup_running');
            delete_transient('cache_warmup_in_progress');
            delete_transient('berqwp_doing_cache_warmup');

            do_action('berqwp_flush_all_cache');
            // do_action('rt_nginx_helper_purge_all');
        }

        function add_clear_cache_link_to_admin_bar()
        {
            global $wp_admin_bar;

            // Check if the user has the capability to clear the cache (adjust the capability as needed)
            if (current_user_can('manage_options')) {
                $wp_admin_bar->add_menu(
                    array(
                        'id' => 'berqWP',
                        'title' => 'BerqWP',
                        'href' => get_admin_url() . '/admin.php?page=berqwp',
                    )
                );

                // Add the sub-menu item
                $wp_admin_bar->add_menu(
                    array(
                        'parent' => 'berqWP',
                        // ID of the parent menu item
                        'id' => 'flush-cache',
                        'title' => 'Flush cache',
                        'href' => wp_nonce_url(admin_url('admin-post.php?action=clear_cache'), 'clear_cache_action'),
                        'meta' => array(
                            'class' => 'clear-cache-link',
                            'title' => 'Clear BerqWP cache',
                        ),
                    )
                );

                if (!is_admin()) {
                    // Add the sub-menu item
                    $wp_admin_bar->add_menu(
                        array(
                            'parent' => 'berqWP',
                            // ID of the parent menu item
                            'id' => 'purge-page',
                            'title' => 'Purge this page',
                            'href' => wp_nonce_url(admin_url('admin-post.php?action=berq_purge_page&uri=' . urlencode($_SERVER['REQUEST_URI'])), 'berq_purge_page_action'),
                            'meta' => array(
                                'class' => 'purge-page-link',
                                'title' => 'Clear this page cache',
                            ),
                        )
                    );

                }
            }
        }


        function html_cache()
        {
            // Check if the current user is logged in or if it's a POST request
            if (is_user_logged_in() || $_SERVER['REQUEST_METHOD'] === 'POST') {
                return;
            }

            if (isset($_GET['generating_critical_css'])) {
                return;
            }

            $slug_uri = $_SERVER['REQUEST_URI'];

            // if wordpress is installed in a sub directory
            if (berqwp_is_sub_dir_wp()) {
                // Parse strings to extract paths
                $path1 = explode('/', parse_url(home_url(), PHP_URL_PATH));
                $path2 = explode('/', parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH));

                // Find the common part of the paths
                $commonPath = implode('/', array_intersect($path1, $path2));

                // Subtract the common part from the first string
                $slug_uri = str_replace($commonPath, '', $_SERVER['REQUEST_URI']);
            }

            // Return if page is excluded from cache
            $pages_to_exclude = get_option('berq_exclude_urls', []);

            if (in_array(home_url() . $slug_uri, $pages_to_exclude)) {
                return;
            }

            // Remove ignored params from the slug
            $slug = berqwp_remove_ignore_params($slug_uri);

            if (isset($_GET['creating_cache'])) {
                return;
            }

            if (get_option('berqwp_enable_sandbox') == 1 && isset($_GET['berqwp'])) {
                $slug = explode('?berqwp', $slug_uri)[0];
            } elseif (get_option('berqwp_enable_sandbox') == 1 && !isset($_GET['creating_cache'])) {
                return;
            }


            // Attempt to retrieve the cached HTML from the cache directory
            $cache_directory = optifer_cache . '/html/';

            // Generate a unique cache key based on the current page URL
            $cache_key = md5($slug);
            $cache_file = $cache_directory . $cache_key . '.html';
            $status_code = http_response_code();

            if ($status_code !== 200) {
                return;
            }

            if (!isset($_GET['creating_cache'])) {
                if (strpos($slug, '?creating_cache') === false) {
                    $page_modified_time = berqwp_get_last_modified_timestamp();
                    $cache_life_span = time() - (20 * 60 * 60);

                    // Hook to modify cache lifespan
                    $cache_life_span = apply_filters('berqwp_cache_lifespan', $cache_life_span);

                    if (
                        !file_exists($cache_file) ||
                        (file_exists($cache_file) && filemtime($cache_file) < $cache_life_span) ||
                        (file_exists($cache_file) && !empty($page_modified_time) && $page_modified_time > filemtime($cache_file))
                    ) {

                        global $berq_log;
                        $startTime = microtime(true);

                        $args = array(
                            'blocking' => false,
                            'timeout' => 0.01,
                            'body' => array(
                                'slug' => $slug,
                            ),
                        );

                        wp_remote_post(get_site_url() . '/wp-json/optifer/v1/warmup-cache', $args);

                        // Get end time
                        $endTime = microtime(true);

                        // Calculate runtime in milliseconds
                        $runtime = ($endTime - $startTime) * 1000;

                        $berq_log->info("Requesting cache for $slug took $runtime ms.");

                    }

                }
            }

            // If the cached HTML file exists, serve it and stop further execution
            if (!isset($_GET['creating_cache']) && file_exists($cache_file)) {
                $lastModified = filemtime($cache_file);
                $etag = md5_file($cache_file);
                header('ETag: ' . $etag);

                // if (isset($_SERVER['HTTP_IF_NONE_MATCH']) && $_SERVER['HTTP_IF_NONE_MATCH'] === $etag) {
                //     // The client's cache is still valid, respond with a 304 Not Modified
                //     header('HTTP/1.1 304 Not Modified');
                // } else {
                //     // Set other headers and output the content as before
                //     header('Last-Modified: ' . gmdate('D, d M Y H:i:s', filemtime($cache_file)) . ' GMT');
                //     header('Cache-Control: public, max-age=3600');
                //     readfile($cache_file);
                //     exit();
                // }

                // Check if the client has a cached copy and if it's still valid using ETag
                if (isset($_SERVER['HTTP_IF_NONE_MATCH']) && $_SERVER['HTTP_IF_NONE_MATCH'] === $etag) {
                    // The client's cache is still valid based on ETag, respond with a 304 Not Modified
                    header('HTTP/1.1 304 Not Modified');
                    header('Expires: ' . gmdate('D, d M Y H:i:s') . ' GMT');
                    header('Cache-Control: no-cache, must-revalidate');

                } else {
                    // Check if the client has a cached copy and if it's still valid using Last-Modified
                    if (isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) && strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']) >= $lastModified) {
                        // The client's cache is still valid based on Last-Modified, respond with a 304 Not Modified
                        header('HTTP/1.1 304 Not Modified');
                        header('Expires: ' . gmdate('D, d M Y H:i:s') . ' GMT');
                        header('Cache-Control: no-cache, must-revalidate');

                    } else {
                        // Set other headers and output the content as before
                        header('Last-Modified: ' . gmdate('D, d M Y H:i:s', $lastModified) . ' GMT');
                        header('Cache-Control: public, max-age=86400'); // Cache for one day
                        readfile($cache_file);
                        exit();
                    }
                }
            }

            return;

        }

        function warmup_cache($page)
        {
            global $berq_log;
            $berq_log->info("* * * * * * * * *");
            $berq_log->info("* Cache Warmup");
            $berq_log->info("* * * * * * * * *");

            $posts_per_batch = $this->max_page_per_batch;

            // if (get_transient('berqwp_doing_cache_warmup')) {
            //     return;
            // }

            // set_transient('berqwp_doing_cache_warmup', true, 120);

            // Set a transient to indicate that the cache warmup is in progress
            set_transient('cache_warmup_in_progress', true, 120);

            // warmup_cache_by_slug('/');

            // $post_types = get_post_types(array('publicly_queryable' => true), 'names');
            $post_types = get_option('berqwp_post_type_names');

            // API endpoint URL
            $api_endpoint = 'https://boost.berqwp.com/photon/';

            if (get_site_url() == 'http://berq-test.local') {
                $api_endpoint = 'http://dev-berqwp.local/photon/';
            }

            // Modify photon engine endpoint for testing purposes
            $api_endpoint = apply_filters( 'berqwp_photon_endpoint', $api_endpoint );

            // $post_types[] = 'product';
            $args = array(
                'post_type' => $post_types,
                'posts_per_page' => $posts_per_batch,
                'paged' => $page,
            );

            $query = new WP_Query($args);

            $httpMulti = new bwp_multi_http();

            $post_data = berqwp_get_page_params('/');
            $httpMulti->addRequest('POST', $api_endpoint, $post_data);

            // Loop through all posts and make a GET request
            while ($query->have_posts()) {
                $query->the_post();

                // Get the post URL and make a GET request
                $url = trailingslashit(get_permalink());
                $slug = str_replace(home_url(), '', $url);
                $post_data = berqwp_get_page_params($slug);

                // Attempt to retrieve the cached HTML from the cache directory
                $cache_directory = optifer_cache . '/html/';

                // Generate a unique cache key based on the current page URL
                $cache_key = md5($slug);
                $cache_file = $cache_directory . $cache_key . '.html';

                $page_modified_time = berqwp_get_last_modified_timestamp();
                $cache_life_span = time() - (20 * 60 * 60);

                // Hook to modify cache lifespan
                $cache_life_span = apply_filters('berqwp_cache_lifespan', $cache_life_span);

                if (
                    !file_exists($cache_file) ||
                    (file_exists($cache_file) && filemtime($cache_file) < $cache_life_span) ||
                    (file_exists($cache_file) && !empty($page_modified_time) && $page_modified_time > filemtime($cache_file))
                ) {
                    $httpMulti->addRequest('POST', $api_endpoint, $post_data);
    
                    $berq_log->info("Requesting cache for $slug");

                }


            }

            $responses = $httpMulti->execute();

            wp_reset_postdata();

            delete_transient('cache_warmup_in_progress');
            delete_transient('berqwp_doing_cache_warmup');

        }


        function cache_warmup_admin_notice()
        {


            if (get_transient('cache_warmup_in_progress')) {
                ?>
                <div class="notice notice-info is-dismissible">
                    <p>
                        <?php
                        _e('Cache warmup is in progress... This process could take some time depending on the number of pages on your website.', 'searchpro');
                        ?>
                    </p>
                </div>
                <?php
            }

            if (!get_transient('cache_warmup_in_progress') && isset($_GET['berq_warmingup'])) {
                ?>
                <div class="notice notice-info is-dismissible">
                    <p>
                        <?php _e('BerqWP is starting cache warmup. Please wait.', 'searchpro'); ?>
                    </p>
                </div>
                <?php
            }

            if (get_transient('berq_cache_cleared_notice')) {
                delete_transient('berq_cache_cleared_notice');
                ?>
                <div class="notice notice-success is-dismissible">
                    <p>
                        <?php _e('The cache has been cleared. Our automatic cache warm-up system will generate the cache. Alternatively, you can
                        visit any page to create its cache immediately.', 'searchpro'); ?>
                    </p>
                </div>
                <?php
            }
        }

    }

    $cache = new berqCache();

}
