<?php

class berqIntegrations
{
    function __construct()
    {
        // Triggers when flushing all cache
        add_action('berqwp_flush_all_cache', [$this, 'flush_all_cache']);

        // Triggers when storing page cache
        add_action('berqwp_stored_page_cache', [$this, 'flush_all_cache']);
    }

    function flush_all_cache()
    {

        // Redis Object Cache Pro
        $this->flush_object_cache_pro();

        // Nginx Helper
        $this->flush_nginx_helper();

        // Flush Nginx webserver cache
        $this->flush_nginx_cache();

        // Cloudflare page cache
        $this->flush_cloudflare_page_cache();
    }

    function flush_object_cache_pro()
    {
        if (function_exists('wp_cache_flush')) {
            wp_cache_flush(); // Clear the entire object cache.
        }
    }

    function flush_nginx_helper()
    {
        if (class_exists('Nginx_Helper')) {
            do_action('rt_nginx_helper_purge_all');
        }
    }

    function flush_nginx_cache()
    {
        $is_nginx = stripos($_SERVER['SERVER_SOFTWARE'], 'nginx') !== false;

        if ($is_nginx) {
            // Clear nginx cache

            // Common cache dirs
            $dirs = ['/var/run/nginx-cache', '/var/cache/nginx', '/var/lib/nginx/cache'];

            foreach ($dirs as $dir) {
                berqwp_unlink_recursive($dir);
            }

        }
    }

    function flush_cloudflare_page_cache() {
        if (class_exists('SW_CLOUDFLARE_PAGECACHE') && isset($GLOBALS['sw_cloudflare_pagecache'])) {
            $GLOBALS['sw_cloudflare_pagecache']->objects['cache_controller']->purge_all();
        }
    }

}
$berqIntegrations = new berqIntegrations();