<?php
if (!defined('ABSPATH'))
    exit;


function berqwp_is_slug_excludable($slug)
{
    if (empty($slug)) {
        return true;
    }

    $exclude_items = ["elementor_library=", "add_to_wishlist=", "robots.txt", ".html", ".php", "run_warmup="];

    foreach ($exclude_items as $item) {
        if (strpos($slug, $item) !== false) {
            return true;
        }
    }

    return false;
}

function berqwp_get_page_params($slug, $is_forced = false) {
    if (empty($slug)) {
        return;
    }

    $url = home_url() . $slug;
    $slug_md5 = md5($slug);

    $cache_directory = optifer_cache . '/html/';
    $cache_file = $cache_directory . $slug_md5 . '.html';
    $key = uniqid();

    if (!file_exists($cache_file)) {
        $key = '';
    }

    $optimization_mode = get_option('berq_opt_mode');

    if ($optimization_mode == 4) {
        $optimization_mode = 'aggressive';
    } elseif ($optimization_mode == 3) {
        $optimization_mode = 'blaze';
    } elseif ($optimization_mode == 2) {
        $optimization_mode = 'medium';
    } elseif ($optimization_mode == 1) {
        $optimization_mode = 'basic';
    }

    // Data to send as POST parameters
    $post_data = array(
        'license_key'                   => get_option('berqwp_license_key'),
        'page_url'                      => $url,
        'page_slug'                     => $slug,
        'site_url'                      => home_url(),
        'webp_max_width'                => (int) get_option('berqwp_webp_max_width'),
        'webp_quality'                  => (int) get_option('berqwp_webp_quality'),
        'img_lazyloading'               => get_option('berqwp_image_lazyloading'),
        'youtube_lazyloading'           => get_option('berqwp_lazyload_youtube_embed'),
        'js_mode'                       => get_option('berqwp_javascript_execution_mode'),
        'key'                           => $key,
        'interaction_delay'             => get_option('berqwp_interaction_delay'),
        'cache_js'                      => true,
        'use_cdn'                       => get_option('berqwp_enable_cdn'),
        'opt_mode'                      => $optimization_mode,
        'disable_webp'                  => get_option('berqwp_disable_webp'),
        'js_css_exclude_urls'           => get_option('berq_exclude_js_css', []),
        'preload_fontfaces'             => get_option('berqwp_preload_fontfaces'),
        // 'mobile_lcp'                 => json_encode($mobile_lcp),
        // 'desktop_lcp'                => json_encode($desktop_lcp),
        'version'                       => BERQWP_VERSION
    );

    if (defined('BERQ_STAGING')) {
        $post_data['run_queue'] = 1;
    }

    if ($is_forced) {
        $post_data['force'] = true;
    }

    return $post_data;


}

function warmup_cache_by_slug($slug, $is_forced = false)
{
    if (empty($slug)) {
        return;
    }

    if (berqwp_is_slug_excludable($slug)) {
        return;
    }

    // API endpoint URL
    $api_endpoint = 'https://boost.berqwp.com/photon/';
    
    if (get_site_url() == 'http://berq-test.local') {
        $api_endpoint = 'http://dev-berqwp.local/photon/';
    }

    // Modify photon engine endpoint for testing purposes
    $api_endpoint = apply_filters( 'berqwp_photon_endpoint', $api_endpoint );

    $post_data = berqwp_get_page_params($slug, $is_forced);
    
    // Set up the request arguments
    $args = array(
        'body'              => $post_data,  // Pass the POST data here
        'method'            => 'POST',
        // 'blocking'          => false,
        'timeout'           => 0.01,
        'headers' => array(
            'Content-Type' => 'application/x-www-form-urlencoded', // Adjust content type if needed
        ),
    );

    // Send the POST request
    $response = wp_remote_post($api_endpoint, $args);

    // Check for errors and handle the response
    if (is_wp_error($response)) {
        // There was an error with the request
        error_log('Error: ' . $response->get_error_message());
    }


}

function berq_is_localhost()
{
    $whitelist = array('127.0.0.1', '::1');

    if (in_array($_SERVER['REMOTE_ADDR'], $whitelist)) {
        return true;
    }

    return false;
}

function berqwp_remove_ignore_params($slug)
{
    // List of tracking parameters to remove
    $tracking_params = get_option('berq_ignore_urls_params', []);

    $tracking_params = apply_filters( 'berqwp_ignored_urls_params', $tracking_params );

    // Parse the provided slug
    $url_parts = parse_url($slug);
    
    // Get the current URL parameters
    $url_params = array();
    if (isset($url_parts['query'])) {
        parse_str($url_parts['query'], $url_params);
    }
    
    // Remove specified tracking parameters from the URL
    foreach ($tracking_params as $param) {
        $param = trim($param);
        if (isset($url_params[$param])) {
            unset($url_params[$param]);
        }
    }

    // Build the new query string
    $new_query_string = http_build_query($url_params);

    // Reconstruct the URL with the new query string
    $new_slug = $url_parts['path'];
    if (!empty($new_query_string)) {
        $new_slug .= '?' . $new_query_string;
    }

    return $new_slug;
}

function berqwp_is_sub_dir_wp()
{
    // remove http
    $site_url = explode('//', home_url())[1];
    $break_slash = explode('/', $site_url);

    return count($break_slash) > 1;
}

function berqwp_current_page_cache_file()
{
    $slug_uri = $_SERVER['REQUEST_URI'];

    // if wordpress is installed in a sub directory
    if (berqwp_is_sub_dir_wp()) {
        // Parse strings to extract paths
        $path1 = explode('/', parse_url(home_url(), PHP_URL_PATH));
        $path2 = explode('/', parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH));

        // Find the common part of the paths
        $commonPath = implode('/', array_intersect($path1, $path2));

        // Subtract the common part from the first string
        $slug_uri = str_replace($commonPath, '', $_SERVER['REQUEST_URI']);
    }

    // Return if page is excluded from cache
    $pages_to_exclude = get_option('berq_exclude_urls', []);

    if (in_array(get_site_url() . $slug_uri, $pages_to_exclude)) {
        return;
    }


    $slug = berqwp_remove_ignore_params($slug_uri);

    if (isset($_GET['creating_cache'])) {
        return;
    }

    if (get_option('berqwp_enable_sandbox') == 1 && isset($_GET['berqwp'])) {
        $slug = explode('?berqwp', $slug_uri)[0];
    } elseif (get_option('berqwp_enable_sandbox') == 1 && !isset($_GET['creating_cache'])) {
        return;
    }


    // Attempt to retrieve the cached HTML from the cache directory
    $cache_directory = optifer_cache . '/html/';

    // Generate a unique cache key based on the current page URL
    $cache_key = md5($slug);
    $cache_file = $cache_directory . $cache_key . '.html';

    return $cache_file;

}

function berqwp_get_LCP_details($url, $device = 'mobile')
{
    $google_pagespeed_api_url = "https://www.googleapis.com/pagespeedonline/v5/runPagespeed?url=$url&strategy=$device ";

    // Send a GET request to the Google PageSpeed Insights API
    $response = wp_remote_get($google_pagespeed_api_url, array('timeout' => 60));

    if (is_wp_error($response)) {
        return 'Error: ' . $response->get_error_message();
    }

    // Convert the JSON response to a PHP array
    $body = wp_remote_retrieve_body($response);
    $output = json_decode($body, true);

    // Get the LCP data        
    return $output['lighthouseResult']['audits']['largest-contentful-paint-element']['details']['items'][0]['items'][0]['node'];
}

function berqwp_enable_object_cache($enable) {
    global $berq_log;
    $berq_log->info("Updating wp-config.php");

    // Specify the wp-config.php file path
    $wp_config_file = ABSPATH . 'wp-config.php';

    // Read the contents of wp-config.php
    $wp_config_content = file_get_contents($wp_config_file);

    // Find the position of the first PHP tag using a regular expression
    preg_match('/<\?php/', $wp_config_content, $matches, PREG_OFFSET_CAPTURE);

    // Check if the PHP opening tag exists
    if (!empty($matches)) {
        $first_php_comment_position = $matches[0][1] + 5; // Move past the length of '<?php'

        // Check if the WP_CACHE definition exists in the file
        if (strpos($wp_config_content, "define('WP_CACHE'") === false && strpos($wp_config_content, "define( 'WP_CACHE' ") === false) {
            // If not, add the definition right after the opening PHP tag
            $wp_config_content = substr_replace($wp_config_content, "\n"
                                . "// Enable or disable BerqWP object cache\n"
                                . "define('WP_CACHE', " . ($enable ? 'true' : 'false') . ");\n",
                                $first_php_comment_position, 0);
        } else {
            // Otherwise, enable or disable the existing definition
            // $wp_config_content = preg_replace(
            //     "/define\('WP_CACHE', [^\n]*\);/",
            //     "define('WP_CACHE', " . ($enable ? 'true' : 'false') . ");",
            //     $wp_config_content
            // );

            $wp_config_content = preg_replace(
                "/define\(\s*'WP_CACHE'\s*,\s*[^\n]*\);/",
                "define('WP_CACHE', " . ($enable ? 'true' : 'false') . ");",
                $wp_config_content
            );
        }

        // Write the modified content back to wp-config.php
        file_put_contents($wp_config_file, $wp_config_content);
    } else {

        global $berq_log;
        $berq_log->error("Error: PHP opening tag not found in wp-config.php");
        
    }
}

// Copied from Nginx Helper plugin
function berqwp_unlink_recursive( $dir ) {

    if ( ! is_dir( $dir ) ) {
        return;
    }

    $dh = opendir( $dir );

    if ( ! $dh ) {
        return;
    }

    // phpcs:ignore -- WordPress.CodeAnalysis.AssignmentInCondition.FoundInWhileCondition -- Variable assignment required for recursion.
    while ( false !== ( $obj = readdir( $dh ) ) ) {

        if ( '.' === $obj || '..' === $obj ) {
            continue;
        }

        if ( ! @unlink( $dir . '/' . $obj ) ) { // phpcs:ignore WordPress.PHP.NoSilencedErrors.Discouraged
            berqwp_unlink_recursive( $dir . '/' . $obj, false );
        }
    }

    closedir( $dh );
}

function berqwp_get_last_modified_timestamp() {
    global $post;

    // Check if it's a single post or page
    if (is_singular()) {
        return get_the_modified_time('U', $post->ID); // 'U' format parameter returns Unix timestamp
    }

    // Check if it's a taxonomy term
    if (is_tax() || is_category() || is_tag()) {
        $term = get_queried_object(); // Get the current term object

        // For tags, get the last modified date of the most recent post associated with the tag
        if (is_tag()) {
            $args = array(
                'tag_id' => $term->term_id,
                'posts_per_page' => 1,
                'orderby' => 'modified',
                'order' => 'DESC',
                'fields' => 'ids', // Return only post IDs to reduce overhead
            );
            $posts = get_posts($args);
            if ($posts) {
                $latest_post_id = $posts[0];
                return get_the_modified_time('U', $latest_post_id); // 'U' format parameter returns Unix timestamp
            }
        }

        // For category archives, get the last modified date of the most recent post within the category
        if (is_category()) {
            $args = array(
                'category' => $term->term_id,
                'posts_per_page' => 1,
                'orderby' => 'modified',
                'order' => 'DESC',
                'fields' => 'ids', // Return only post IDs to reduce overhead
            );
            $posts = get_posts($args);
            if ($posts) {
                $latest_post_id = $posts[0];
                return get_the_modified_time('U', $latest_post_id); // 'U' format parameter returns Unix timestamp
            }
        }

        return strtotime($term->modified); // Convert modified date to timestamp
    }

    // Check if it's an archive
    if (is_archive()) {
        // For other archives
        $archive_id = get_queried_object_id(); // Get the ID of the current archive
        $archive = get_post($archive_id); // Get the archive post object
        return strtotime($archive->post_modified); // Convert modified date to timestamp
    }

    // For other cases (fallback)
    return false;
}



